import argparse
from os.path import join

from core import shamir_decode, shamir_encode


def main():
    parser = argparse.ArgumentParser("Shamir's Secret Sharing")
    parser.set_defaults(mode='None')

    subparsers = parser.add_subparsers()

    split = subparsers.add_parser("split")
    split.add_argument("file", help="File to split into multiple files")
    split.add_argument("-n", type=int, required=True,
                       help="Number of parts to split the file.")
    split.add_argument("-k", type=int, required=True,
                       help="Expected number of parts needed to recover the original file.")
    split.add_argument("-o", "--output", default=".",
                       help="Output directory of all the files.")
    split.set_defaults(mode='split')

    recover = subparsers.add_parser("recover")
    recover.add_argument("directory")
    recover.add_argument("-p", "--prefix",
                         help="Read parts from files in directory with this prefix.")
    recover.add_argument("-o", "--output",
                         help="Output file to save the recovered file.")
    recover.set_defaults(mode='recover')

    args = parser.parse_args()

    if args.mode == 'split':
        with open(args.file, 'rb') as f:
            data = f.read()
        parts = shamir_encode(data, args.k, args.n)

        for part_id, part in enumerate(parts):
            name_file = f'{args.file}.{part_id+1}.shamir'
            path = join(args.output, name_file)
            with open(path, 'wb') as f:
                f.write(part)

    elif args.mode == 'recover':
        pass
    else:
        parser.print_help()


if __name__ == '__main__':
    main()
